@@ -13,7 +13,6 @@
 #include "../common/hash.c"
 #include "tap-interface.h"
 #include <stdlib.h>
-#include <err.h>
 #include "logging.h"
 
 int main(int argc, char *argv[])
@@ -28,9 +27,9 @@ int main(int argc, char *argv[])
 
 	ok1(tdb);
 	key.dsize = strlen("hi");
-	key.dptr = (void *)"hi";
+	key.dptr = discard_const_p(uint8_t, "hi");
 	data.dsize = strlen("world");
-	data.dptr = (void *)"world";
+	data.dptr = discard_const_p(uint8_t, "world");
 
 	ok1(tdb_store(tdb, key, data, TDB_INSERT) == 0);
 	ok1(tdb_check(tdb, NULL, NULL) == 0);
