@@ -1,4 +1,4 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
 
    common events code for signal events
@@ -30,14 +30,22 @@
 #include "tevent_internal.h"
 #include "tevent_util.h"
 
-#define TEVENT_NUM_SIGNALS 64
-
 /* maximum number of SA_SIGINFO signals to hold in the queue.
   NB. This *MUST* be a power of 2, in order for the ring buffer
   wrap to work correctly. Thanks to Petr Vandrovec <petr@vandrovec.name>
   for this. */
 
-#define TEVENT_SA_INFO_QUEUE_COUNT 64
+#define TEVENT_SA_INFO_QUEUE_COUNT 256
+
+size_t tevent_num_signals(void)
+{
+	return TEVENT_NUM_SIGNALS;
+}
+
+size_t tevent_sa_info_queue_count(void)
+{
+	return TEVENT_SA_INFO_QUEUE_COUNT;
+}
 
 struct tevent_sigcounter {
 	uint32_t count;
@@ -172,12 +180,24 @@ static int tevent_common_signal_list_destructor(struct tevent_common_signal_list
 */
 static int tevent_signal_destructor(struct tevent_signal *se)
 {
-	struct tevent_common_signal_list *sl;
-	sl = talloc_get_type(se->additional_data,
-			     struct tevent_common_signal_list);
+	struct tevent_common_signal_list *sl =
+		talloc_get_type_abort(se->additional_data,
+		struct tevent_common_signal_list);
 
 	if (se->event_ctx) {
-		DLIST_REMOVE(se->event_ctx->signal_events, se);
+		struct tevent_context *ev = se->event_ctx;
+
+		DLIST_REMOVE(ev->signal_events, se);
+
+		if (ev->signal_events == NULL && ev->pipe_fde != NULL) {
+			/*
+			 * This was the last signal. Destroy the pipe.
+			 */
+			TALLOC_FREE(ev->pipe_fde);
+
+			close(ev->pipe_fds[0]);
+			close(ev->pipe_fds[1]);
+		}
 	}
 
 	talloc_free(sl);
@@ -313,7 +333,7 @@ struct tevent_signal *tevent_common_add_signal(struct tevent_context *ev,
 		sig_state->oldact[signum] = talloc(sig_state, struct sigaction);
 		if (sig_state->oldact[signum] == NULL) {
 			talloc_free(se);
-			return NULL;			
+			return NULL;
 		}
 		if (sigaction(signum, &act, sig_state->oldact[signum]) == -1) {
 			talloc_free(se);
@@ -357,7 +377,7 @@ int tevent_common_check_signal(struct tevent_context *ev)
 	if (!sig_state || !TEVENT_SIG_PENDING(sig_state->got_signal)) {
 		return 0;
 	}
-	
+
 	for (i=0;i<TEVENT_NUM_SIGNALS+1;i++) {
 		struct tevent_common_signal_list *sl, *next;
 		struct tevent_sigcounter counter = sig_state->signal_count[i];
@@ -406,7 +426,7 @@ int tevent_common_check_signal(struct tevent_context *ev)
 					uint32_t ofs = (counter.seen + j)
 						% TEVENT_SA_INFO_QUEUE_COUNT;
 					se->handler(ev, se, i, 1,
-						    (void*)&sig_state->sig_info[i][ofs], 
+						    (void*)&sig_state->sig_info[i][ofs],
 						    se->private_data);
 					if (!exists) {
 						break;
@@ -470,9 +490,9 @@ int tevent_common_check_signal(struct tevent_context *ev)
 
 void tevent_cleanup_pending_signal_handlers(struct tevent_signal *se)
 {
-	struct tevent_common_signal_list *sl;
-	sl = talloc_get_type(se->additional_data,
-			     struct tevent_common_signal_list);
+	struct tevent_common_signal_list *sl =
+		talloc_get_type_abort(se->additional_data,
+		struct tevent_common_signal_list);
 
 	tevent_common_signal_list_destructor(sl);
 
